//
//  CommonUtil.m
//  TestApp
//
//  Created by sanaeg on 2015/01/23.
//  Copyright (c) 2015年 DaiwaHouse. All rights reserved.
//

#import "CommonUtil.h"

NSString* const kKeySeparate = @".";

@implementation CommonUtil

+ (NSString*) appendingSeparate:(NSString*)element {
    return [element stringByAppendingString:kKeySeparate];
}

+ (NSArray*) componentsSeparatedByString:(NSString*)target {
    return [target componentsSeparatedByString:kKeySeparate];
}

+ (NSString*) componentsJoinedByString:(NSArray*)target {
    return [target componentsJoinedByString:kKeySeparate];
}

//親Elementのパスを取得
+ (NSString*) getParentElementPath:(NSString*)elementPath {
    
    //区切り文字で分割
    NSMutableArray *array = [[elementPath componentsSeparatedByString:kKeySeparate] mutableCopy];
    //一番最後を削除
    [array removeObjectAtIndex:[array count] - 1];
    if ([array count] > 0) {
        //残りの文字列に対して区切り文字を足して結合
        return [array componentsJoinedByString:kKeySeparate];
    } else {
        //何も残ってない場合は親が存在しないので、空文字を返す
        return @"";
    }
}

//ElementのパスからElement名を取得
+ (NSString*) getLastElement:(NSString*)elementPath {
    //区切り文字で分割
    NSMutableArray *array = [[elementPath componentsSeparatedByString:kKeySeparate] mutableCopy];
    //一番最後の文字列を返却
    return [array lastObject];
}

+ (id) getTargetData:(id)data  nest:(NSInteger)nestCount targetPath:(NSString*)targetPath elementPath:(NSString*)elementPath parent:(BOOL)isParent {
    
    if (elementPath == nil || [elementPath length] < 1) {
        return nil;
    }

    id targetData = [data valueForKeyPath:elementPath];
    
    if (targetData == nil) {
        //データが取得できるまで親のパスを遡る
        NSString* parentElementPath = [CommonUtil getParentElementPath:elementPath];
        [self getTargetData:data nest:nestCount + 1 targetPath:targetPath elementPath:parentElementPath parent:isParent];
    } else {
        //遡りが発生している場合
        if (nestCount > 0) {
            //遡ったElementのみ抽出
            NSString* followElement = [targetPath substringFromIndex:[elementPath length]];
            NSArray* followArray = [CommonUtil componentsSeparatedByString:followElement];
            NSInteger followCount = [followArray count];
            //遡ったElementのデータを順に確認し、対象のElementを追加するためのオブジェクトを取得
            for (int i = 0; i < followCount; i++) {
                if ([targetData isKindOfClass:[NSMutableDictionary class]]) {
                    //確認中のオブジェクトがDictionaryの場合
                    //該当Elementのデータを確認対象にする
                    targetData = [targetData valueForKeyPath:followArray[i]];
                } else {
                    //確認中のオブジェクトが配列の場合
                    //最後のデータを確認対象にする
                    targetData = [(NSMutableArray*)targetData lastObject];
                }
            }
        } else {
            //遡りが発生していない場合
            if (isParent && [targetData isKindOfClass:[NSMutableArray class]]) {
                //親データ取得の場合、かつ配列の場合は一番最後のオブジェクトが親になる
                targetData = [(NSMutableArray*)targetData lastObject];
            }
            //上記以外の場合はそのまま返す
        }
    }
    
    //データを追加する親Elementは必ずNSMutableDictionaryになる
    return targetData;
}

@end
