//
//  LocalResponseData.m
//  HousingApiLibrary
//
//  Created by sanaeg on 2015/01/16.
//  Copyright (c) 2015年 DaiwaHouse. All rights reserved.
//

#import "LocalResponse.h"
#import "CommonUtil.h"

@implementation LocalResponse {
    NSObject* responseData;
}


- (id) initWithResponse:(NSObject*)data {
    
    self = [super init];
    
    responseData = data;
    
    return self;
}

- (BOOL) getResponseResult:(NSString*)rootElement arguments:(va_list)args {
    
    //elementのパスを生成
    NSString* elementPath = [self getElementPath:rootElement list:args];
    //データ取得
    id data = [CommonUtil getTargetData:responseData nest:0 targetPath:elementPath elementPath:elementPath parent:NO];

    if (data != nil && [data isKindOfClass:[NSString class]]) {
        return [((NSString*)data) boolValue];
    } else {
        return false;
    }
}

- (id) getResponseData:(NSString*)rootElement arguments:(va_list)args {

    //elementのパスを生成
    NSString* elementPath = [self getElementPath:rootElement list:args];
    
    //指定のパスのデータを取得して呼び元に返す
    return [CommonUtil getTargetData:responseData nest:0 targetPath:elementPath elementPath:elementPath parent:NO];
}

- (id) getResponseDataByKey:(NSString*)keyValue elements:(NSString*)rootElement arguments:(va_list)args {
    
    //elementのパスを生成
    NSString* elementPath = [self getElementPath:rootElement list:args];
    
    if (elementPath == nil) {
        return nil;
    } else {
        //指定のパスの親データを取得
        NSString* parnetElementPath = [CommonUtil getParentElementPath:elementPath];
        id parentData = [CommonUtil getTargetData:responseData nest:0 targetPath:parnetElementPath elementPath:parnetElementPath parent:NO];
        
        if (parentData != nil) {
            if ([parentData isKindOfClass:[NSDictionary class]]) {
                return [self getResponseData:parentData keyValue:keyValue elementPath:elementPath];
            } else if ([parentData isKindOfClass:[NSArray class]]) {
                NSUInteger count = ((NSArray*)parentData).count;
                
                for(int i = 0; i < count - 1; i++) {
                    id retData = [self getResponseData:((NSArray*)parentData)[i] keyValue:keyValue elementPath:elementPath];
                    if (retData != nil) {
                        return retData;
                    }
                }
            }
        }
    }
    
    return nil;
}

- (NSString*) getElementPath:(NSString*)rootElement list:(va_list)args {

    //rootが指定されていない場合はnilを返す
    if (rootElement == nil || rootElement.length < 1) {
        return nil;
    }
    
    NSMutableArray *argsArray = [NSMutableArray array];
    NSString* value = rootElement;
    while (value) {
        //可変長オブジェクトをリストに追加
        [argsArray addObject:value];
        //次の値を取得
        value = va_arg(args, typeof(NSString*));
    }
    //ピリオドで結合
    return [CommonUtil componentsJoinedByString:argsArray];
}

- (id) getResponseData:(id)target keyValue:(NSString*) keyValue elementPath:(NSString*)elementPath {
    NSString* lastPath = [CommonUtil getLastElement:elementPath];
    if (target[lastPath] != nil
        && [target[lastPath] isKindOfClass:[NSString class]]
        && (NSString*)target[lastPath] == keyValue) {
        return target;
    } else {
        return nil;
    }
}

@end
