//
//  LocalXmlPaeserDelegate.m
//  HousingApiLibrary
//
//  Created by sanaeg on 2015/01/16.
//  Copyright (c) 2015年 DaiwaHouse. All rights reserved.
//

#import "LocalXmlPaeserDelegate.h"
#import "CommonUtil.h"

@implementation LocalXmlPaeserDelegate {
    
    NSString* targetElement;
    NSString* targetCharacter;
    
    NSObject* targetData;
}

@synthesize response;

- (id) init {
    
    self = [super init];
    
    response = [NSMutableDictionary dictionary];
    
    return self;
}

//解析開始
- (void) parserDidStartDocument:(NSXMLParser *)parser {
    
}

//要素の開始
- (void) parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {

    if (targetElement == nil) {
        //rootエレメント用のデータ構造を生成
        response[elementName] = [NSMutableDictionary dictionary];
        targetElement = elementName;
    } else {
        NSString* parentElement = targetElement;
        //ElementPathを生成
        if ([targetElement length] > 0) {
            targetElement = [CommonUtil appendingSeparate:targetElement];
        }
        targetElement = [targetElement stringByAppendingString:elementName];
        //親要素に子要素のデータ構造を追加
        [self getTargetData:parentElement];

    }
}

//テキストデータ
- (void) parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
    
    NSString* character = [string stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
    if ([character length] > 0) {
        targetCharacter = character;
    }
}

//要素の終了
- (void) parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {

    if (targetCharacter != nil) {
        if ([targetData isKindOfClass:[NSMutableDictionary class]]) {
            [targetData setValue:targetCharacter forKey:elementName];
        } else {
            ((NSMutableArray*)targetData)[[((NSMutableArray*)targetData) count] - 1] = targetCharacter;
        }
    }
    
    //保持している要素名からデータ取得済みの要素を除外する
    if ([targetElement hasSuffix:elementName]) {
        targetElement = [CommonUtil getParentElementPath:targetElement];
    }
    
    targetCharacter = nil;
}


//解析終了
- (void) parserDidEndDocument:(NSXMLParser *)parser {
    
}


//指定のElementパスのデータを取得
- (id) getElementData:(NSInteger)nestCount elementPath:(NSString*)elementPath {
    //指定のElementPathでデータ取得
    id data = nil;
    
    if ([elementPath length] > 0) {
        data = [CommonUtil getTargetData:response nest:nestCount targetPath:targetElement elementPath:elementPath parent:YES];
    } else {
        //空の場合はルートなので格納オブジェクトそのものを返す
        data = response;
    }

    //データを追加する親Elementは必ずNSMutableDictionaryになる
    return data;
}

//データ格納先のオブジェクトを取得
- (void) getTargetData:(NSString*)parentElement {
    //指定のElementPathでデータ取得
    NSObject* data = [self getElementData:0 elementPath:parentElement];

    //データ構造を生成して親オブジェクトに追加
    NSString* lastElement = [CommonUtil getLastElement:targetElement];
    if ([data isKindOfClass:[NSMutableDictionary class]]) { //親Elementのデータが　Dictionaryの場合
        if ([[(NSMutableDictionary*)data allKeys] containsObject:lastElement]) {
            //既に同Elementの構造が存在する場合
            NSObject* childData = [data valueForKey:lastElement];
            if (![childData isKindOfClass:[NSMutableArray class]]) { //配列でない場合
                //配列に変換
                NSObject* tmpData = [childData copy];
                childData = [[NSMutableArray alloc] initWithObjects:tmpData, nil];
            }
            //末尾に空のデータを追加
            [(NSMutableArray*)childData addObject:[NSMutableDictionary dictionary]];
            //保持しているデータを更新
            [data setValue:childData forKey:lastElement];
            //データを設定するオブジェクトを保持
            targetData = childData;
        } else { //同Elementの構造が存在しない場合
            //新しくデータ構造を生成
            [data setValue:[NSMutableDictionary dictionary] forKey:lastElement];
            targetData = data;
        }
    } else { //親Elementのデータが配列の場合
        //末尾に空のデータを追加
        [(NSMutableArray*)data addObject:[NSMutableDictionary dictionary]];
        //データを設定するオブジェクトを保持
        targetData = data;
    }
}

@end
